import 'package:args/command_runner.dart';
import 'package:mi_service/mi_service.dart';
import 'package:mi_service/src/utils.dart';

import '../command_runner.dart';

class ActionCommand extends Command<void> {
  @override
  final String name = 'action';

  @override
  final String description = 'Trigger action for device.';

  late final String? did;
  late final int? siid;
  late final int? aiid;
  late final List<dynamic> input;

  ActionCommand() {
    argParser
      ..addOption(
        'did',
        abbr: 'd',
        help: 'Device ID.',
        callback: (s) => did = s,
      )
      ..addOption(
        'siid',
        abbr: 's',
        help: 'Serice ID.',
        callback: (s) => siid = s == null ? null : int.tryParse(s),
      )
      ..addOption(
        'aiid',
        abbr: 'a',
        help: 'Action ID.',
        callback: (s) => aiid = s == null ? null : int.tryParse(s),
      )
      ..addMultiOption(
        'input',
        abbr: 'i',
        help: 'List of inputs.',
        callback: (s) => input = s.map<dynamic>(parseValue).toList(),
      );
  }

  dynamic parseValue(String? value) {
    if (value == null) return null;
    if (value == 'true') return true;
    if (value == 'false') return false;

    return int.tryParse(value) ?? double.tryParse(value) ?? value;
  }

  @override
  Future<void> run() async {
    if (did == null || siid == null || aiid == null) {
      logger.e('Option did, siid and aiid are required.');
      printUsage();
      return;
    }

    final credential = (runner as MiServiceCommandRunner).ioCredential;

    logger.i('Trigger service $siid action $aiid of device $did with $input');
    final r = await MiIOService(credential).action(did!, siid!, aiid!, input);
    print(r.isOk ? 'OK' : 'Failed');
  }
}
