import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:mi_service/mi_service.dart';
import 'package:mi_service/src/utils.dart';

class LoginCommand extends Command<void> {
  @override
  final String name = 'login';

  @override
  final String description = 'Login cloud service with Mi account.';

  late final String sid;
  late final String? username;
  String? password;
  late final bool save;

  LoginCommand() {
    argParser
      ..addOption(
        'sid',
        abbr: 's',
        help: 'Service ID.',
        defaultsTo: 'xiaomiio',
        allowed: ['xiaomiio'],
        callback: (s) => sid = s!,
      )
      ..addOption(
        'username',
        abbr: 'u',
        help: 'Username / User ID / Phone.',
        callback: (s) => username = s,
      )
      ..addOption(
        'password',
        abbr: 'p',
        help: 'Password of Mi account.',
        callback: (s) => password = s,
      )
      ..addFlag(
        'save',
        help: 'Save credential to JSON file.',
        defaultsTo: false,
        callback: (s) => save = s,
      );
  }

  @override
  Future<void> run() async {
    if (username == null) {
      logger.e('Option username is required.');
      printUsage();
      return;
    }

    if (password == null) {
      print('Enter Password:');
      password = stdin.readLineSync();
    }

    final credential = await loginMiService(sid, username!, password!);
    final str = jsonEncoder.convert(credential);
    print(str);

    if (save) {
      File('${sid}_service_credential.json').writeAsStringSync(str);
    }
  }
}
