import 'package:args/command_runner.dart';
import 'package:mi_service/mi_service.dart';
import 'package:mi_service/src/utils.dart';

import '../command_runner.dart';

class PropertyCommand extends Command<void> {
  @override
  final String name = 'property';

  @override
  final String description = 'Get / Set device property of device.';

  late final String? did;
  late final int? siid;
  late final int? piid;
  late final dynamic value;

  PropertyCommand() {
    argParser
      ..addOption(
        'did',
        abbr: 'd',
        help: 'Device ID.',
        callback: (s) => did = s,
      )
      ..addOption(
        'siid',
        abbr: 's',
        help: 'Serice ID.',
        callback: (s) => siid = s == null ? null : int.tryParse(s),
      )
      ..addOption(
        'piid',
        abbr: 'p',
        help: 'Property ID.',
        callback: (s) => piid = s == null ? null : int.tryParse(s),
      )
      ..addOption(
        'value',
        abbr: 'v',
        help: 'Value to set.',
        callback: (s) => value = parseValue(s),
      );
  }

  dynamic parseValue(String? value) {
    if (value == null) return null;
    if (value == 'true') return true;
    if (value == 'false') return false;

    return int.tryParse(value) ?? double.tryParse(value) ?? value;
  }

  @override
  Future<void> run() async {
    if (did == null || siid == null || piid == null) {
      logger.e('Option did, siid and piid are required.');
      printUsage();
      return;
    }

    final credential = (runner as MiServiceCommandRunner).ioCredential;
    final service = MiIOService(credential);

    if (value == null) {
      logger.d('Getting service $siid property $piid from device $did.');
      print(await service.getProperty<dynamic>(did!, siid!, piid!));
    } else {
      logger.d('Setting service $siid property $piid of $did to $value.');
      final r = await service.setProperty<dynamic>(did!, siid!, piid!, value);
      print(r ? 'OK' : 'Failed');
    }
  }
}
