import 'dart:convert';
import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:logger/logger.dart';
import 'package:mi_service/mi_service.dart';
import 'package:mi_service/src/utils.dart';

import 'command/action.dart';
import 'command/decode.dart';
import 'command/devices.dart';
import 'command/login.dart';
import 'command/property.dart';

class MiServiceCommandRunner extends CommandRunner<void> {
  MiServiceCommandRunner() : super('miot', 'Cli for handling Mi service API.') {
    argParser.addOption(
      'level',
      abbr: 'l',
      help: 'Log level.',
      allowed: ['v', 'verbose', 'd', 'debug', 'i', 'info'],
      defaultsTo: 'info',
      callback: (level) {
        Logger.level = const <String, Level>{
              'v': Level.verbose,
              'verbose': Level.verbose,
              'd': Level.debug,
              'debug': Level.debug,
              'i': Level.info,
              'info': Level.info,
            }[level] ??
            Level.info;
      },
    );

    addCommand(LoginCommand());
    addCommand(DecodeCommand());
    addCommand(DevicesCommand());
    addCommand(PropertyCommand());
    addCommand(ActionCommand());
  }

  MiServiceCredential get ioCredential {
    final credentialFile = File('xiaomiio_service_credential.json');
    if (!credentialFile.existsSync()) {
      logger.e('Credential for xiaomiio service not found.\n'
          'Please login and save credential first.');
      printUsage();
      exit(1);
    }

    return MiServiceCredential.fromJson(
      jsonDecode(credentialFile.readAsStringSync()),
    );
  }
}
