import 'package:json_annotation/json_annotation.dart';

part 'io.g.dart';

@JsonSerializable(createToJson: false, genericArgumentFactories: true)
class MiIOServiceResponse<T> {
  final int code;
  final String message;
  final T result;

  MiIOServiceResponse({
    required this.code,
    required this.message,
    required this.result,
  });

  factory MiIOServiceResponse.fromJson(Map<String, dynamic> json) =>
      _$MiIOServiceResponseFromJson(json, (v) => v as T);
}

@JsonSerializable(createFactory: false)
class GetPropertyReq {
  final String did;
  final int siid;
  final int piid;

  const GetPropertyReq({
    required this.did,
    required this.siid,
    required this.piid,
  });

  Map<String, dynamic> toJson() => _$GetPropertyReqToJson(this);
}

@JsonSerializable(createToJson: false)
class GetPropertyResp {
  final int code;
  final String did;
  final int siid;
  final int piid;
  final dynamic value;

  const GetPropertyResp({
    required this.code,
    required this.did,
    required this.siid,
    required this.piid,
    required this.value,
  });

  factory GetPropertyResp.fromJson(Map<String, dynamic> json) =>
      _$GetPropertyRespFromJson(json);

  bool get isOk => code == 0;
}

@JsonSerializable(createFactory: false, genericArgumentFactories: true)
class SetPropertyReq<T> {
  final String did;
  final int siid;
  final int piid;
  final T value;

  const SetPropertyReq({
    required this.did,
    required this.siid,
    required this.piid,
    required this.value,
  });

  Map<String, dynamic> toJson() =>
      _$SetPropertyReqToJson<T>(this, (value) => value as Object);
}

@JsonSerializable(createToJson: false)
class SetPropertyResp {
  final int code;
  final String did;
  final int siid;
  final int piid;

  const SetPropertyResp({
    required this.code,
    required this.did,
    required this.siid,
    required this.piid,
  });

  factory SetPropertyResp.fromJson(Map<String, dynamic> json) =>
      _$SetPropertyRespFromJson(json);

  bool get isOk => code == 0;
}

@JsonSerializable(createToJson: false)
class ActionResp {
  final int code;
  final String did;
  final int siid;
  final int aiid;

  const ActionResp({
    required this.code,
    required this.did,
    required this.siid,
    required this.aiid,
  });

  factory ActionResp.fromJson(Map<String, dynamic> json) =>
      _$ActionRespFromJson(json);

  bool get isOk => code == 0;
}
